/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.component.rt.ProviderReader;
import org.bluestemsoftware.specification.eoa.ext.idl.IDLDocument;

/**
 * Reads <code>Component</code> definition(s) from an IDL document and adds each to
 * <code>ComponentDeployment</code>.
 */
public interface ComponentReader {

    /**
     * Reads <code>Component</code> definition(s) manifested within provided IDL document
     * and adds each to <code>ComponentDeployment</code>.
     * <p>
     * Note that if <code>ComponentReader</code> is also a {@link ProviderReader}, i.e.
     * provider definition is inlined within IDL document, e.g. a schema document or a policy
     * expression, in addition to adding component definition to deployment, a compliant
     * implementation MUST also add all provider definitions read from document to the
     * deployment as well.
     * 
     * @param componentContext
     *        Models context within which components are deployed and to which parsed
     *        definitions MUST be added by reader.
     *        <p>
     *        Note that implementations MUST use the provided context to resolve entities which
     *        are referenced by components, if any.
     * @param idlDocument
     *        Target <code>IDLDocument</code> from which component definition(s) are read.
     *        <p>
     *        Note that <code>ComponentReader</code> implementations MUST read all top-level
     *        component definitions defined within target document and all top-level
     *        definitions defined within document(s) included by target document, if any.
     *        Imported component definition(s) MUST NOT be read unless component is directly
     *        referenced by a top-level component defined within target document (or an
     *        included document) or transitively referenced by a top-level component defined
     *        within target document (or an included document).
     * 
     * @throws DeploymentException
     *         If an error occurs reading component definition(s).
     * 
     * @see ComponentDeployment#addComponent(RootComponent)
     * @see ComponentDeployment#addProvider(RootComponentRT)
     */
    public void readComponents(ComponentContext componentContext, IDLDocument idlDocument) throws DeploymentException;

}
