/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.binding;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier;
import org.bluestemsoftware.specification.eoa.component.NestedComponent;
import org.bluestemsoftware.specification.eoa.component.policy.rt.ElementPolicy;
import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier.Scheme;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceAction.Direction;


public abstract class BindingMessageReference extends NestedComponent<Binding, BindingOperation> implements BindingAction {

    protected Direction direction;
    protected ElementPolicy elementPolicy;
    
    public BindingMessageReference(Direction direction, ElementPolicy elementPolicy) {
        super();
        if (direction == null) {
            throw new IllegalArgumentException("direction null");
        } else {
            this.direction = direction;
        }
        this.elementPolicy = elementPolicy;
    }
    
    /**
     * Returns <code>Direction</code> defined on abstract action to which this concrete
     * action is bound.
     * 
     * @return Direction
     */
    public Direction getDirection() {
        return direction;
    }
    
    /**
     * Gets <code>ElementPolicy</code> attached to component.
     * 
     * @return ElementPolicy or <code>null</code> if no policy attached.
     */
    public ElementPolicy getElementPolicy() {
        return elementPolicy;
    }
    
    /**
     * Identity method which MUST be overriden by implementation. When an attempt is made to
     * import a <code>Binding</code> with the same name as a currently loaded
     * <code>Binding</code>, <code>Binding</code> components are compared for equality.
     * <p>
     * This method is used for error reporting and MUST detail the values which were used to
     * test for equalitiy.
     * 
     * @return String
     */
    public String toString() {
        return null;
    }

    /**
     * Identity method which MUST be overriden by implementation. When an attempt is made to
     * import a <code>Binding</code> with the same name as a currently loaded
     * <code>Binding</code>, <code>Binding</code> components are compared for equality.
     * 
     * @return String
     */
    public int hashCode() {
        return -1;
    }

    /**
     * Identity method which MUST be overriden by implementation. When an attempt is made to
     * import a <code>Binding</code> with the same name as a currently loaded
     * <code>Binding</code>, <code>Binding</code> components are compared for equality.
     * 
     * @return da
     */
    public boolean equals(Object obj) {
        return false;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.Component#getFragmentIdentifier()
     */
    public FragmentIdentifier getFragmentIdentifier() {
        if (fragmentIdentifier == null) {
            QName rootName = getRootComponent().getName();
            StringBuffer pointerPart = new StringBuffer();
            pointerPart.append(rootName.getLocalPart());
            pointerPart.append("/");
            pointerPart.append("xmlns(ns=");
            pointerPart.append(((BindingOperation)parent).getName().getNamespaceURI());
            pointerPart.append(")");
            pointerPart.append("ns:" + ((BindingOperation)parent).getName().getLocalPart());
            pointerPart.append("/");
            pointerPart.append(direction == Direction.IN ? "In" : "Out");
            fragmentIdentifier = new FragmentIdentifier(rootName.getNamespaceURI(), Scheme.EOA, "bindingMessageReference", pointerPart.toString());
        }
        return fragmentIdentifier;
    }
    
    void setParent(BindingOperation parent) {
        this.parent = parent;
    }
    
}