/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.intrface.rt;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.component.binding.UnderlyingProtocol;
import org.bluestemsoftware.specification.eoa.component.engine.rt.EndpointReference;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceFault;
import org.bluestemsoftware.specification.eoa.component.message.InterfaceMessage;
import org.bluestemsoftware.specification.eoa.component.message.rt.Message;

/**
 * 'Thrown' by an <code>Engine</code> instance to indicate the occurrence of a 'business'
 * related error which occured while processing an <code>Action</code>.
 * <p>
 * Models a runtime instance of {@link InterfaceFault}, i.e. which implies all instances of
 * <code>BusinessFault</code> MUST be declared.
 * 
 * @see UnderlyingProtocol#sendAction(EndpointReference, ActionContext)
 */
public class BusinessFault extends Exception implements ApplicationFault {

    private static final long serialVersionUID = 1L;

    public static String REASON = "A business related error occurred while processing action.";

    private QName faultName;
    private Message faultMessage;

    /**
     * Constructs fault.
     * 
     * @param application within which fault occurred.
     * @param faultName
     *        qualified name of fault.
     * @param faultMessage
     *        A runtime instance of {@link InterfaceMessage} of type defined by
     *        {@link InterfaceFault}
     * 
     */
    public BusinessFault(QName faultName, Message faultMessage) {
        super(REASON);
        if (faultName == null) {
            throw new IllegalArgumentException("faultName null");
        }
        if (faultMessage == null) {
            throw new IllegalArgumentException("faultMessage null");
        }
        this.faultName = faultName;
        this.faultMessage = faultMessage;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.intrface.rt.ApplicationFault#getFaultName()
     */
    public final QName getFaultName() {
        return faultName;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.intrface.rt.ApplicationFault#toMessage()
     */
    public Message toMessage() {
        return faultMessage;
    }
    
    /*
     * (non-Javadoc)
     * @see java.lang.Throwable#toString()
     */
    @Override
    public String toString() {
        return faultMessage.toString();
    }

}
