/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.policy.rt;

import java.util.Set;

import org.bluestemsoftware.specification.eoa.component.application.rt.RoleReference;
import org.bluestemsoftware.specification.eoa.component.binding.Binding;
import org.bluestemsoftware.specification.eoa.component.engine.rt.EndpointReference;
import org.bluestemsoftware.specification.eoa.component.intrface.Interface;
import org.bluestemsoftware.specification.eoa.component.service.Endpoint;

/**
 * Aggregates {@link ElementPolicy} with policy subject ENDPOINT. Policy attached to the
 * following components imply policy subject ENDPOINT:
 * <p>
 * {@link Endpoint}
 * <p>
 * {@link Binding}
 * <p>
 * {@link Interface}
 * <p>
 * {@link EndpointReference}
 * <p>
 * {@link RoleReference}
 */

public final class EndpointPolicy extends EffectivePolicy {

    public EndpointPolicy(PolicyExpression expression) {
        super(expression);
    }

    @Override
    public PolicySubject getPolicySubject() {
        return PolicySubject.ENDPOINT;
    }

    @Override
    public EffectivePolicy filter(Set<String> policyVocabularyNamespaces) {
        if (policyVocabularyNamespaces == null) {
            return this;
        }
        PolicyExpression filteredExpression = expression.filterExpression(policyVocabularyNamespaces);
        if (filteredExpression != null) {
            return new EndpointPolicy(filteredExpression);
        } else {
            return null;
        }
    }

}