/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.policy.rt;

import java.util.Set;

import org.bluestemsoftware.specification.eoa.component.binding.BindingFault;
import org.bluestemsoftware.specification.eoa.component.binding.BindingFaultReference;
import org.bluestemsoftware.specification.eoa.component.binding.BindingMessageReference;
import org.bluestemsoftware.specification.eoa.component.engine.rt.EndpointActionReference;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceFault;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceFaultReference;
import org.bluestemsoftware.specification.eoa.component.intrface.InterfaceMessageReference;
import org.bluestemsoftware.specification.eoa.component.message.InterfaceMessage;

/**
 * Aggregates {@link ElementPolicy} with policy subject MESSAGE. Policy attached to the
 * following components imply policy subject MESSAGE:
 * <p>
 * {@link InterfaceMessage}
 * <p>
 * {@link InterfaceMessageReference}
 * <p>
 * {@link InterfaceFault}
 * <p>
 * {@link InterfaceFaultReference}
 * <p>
 * {@link BindingFault}
 * <p>
 * {@link BindingFaultReference}
 * <p>
 * {@link EndpointActionReference}
 * <p>
 * {@link BindingMessageReference}
 */

public final class MessagePolicy extends EffectivePolicy {

    public MessagePolicy(PolicyExpression expression) {
        super(expression);
    }

    @Override
    public PolicySubject getPolicySubject() {
        return PolicySubject.MESSAGE;
    }

    @Override
    public EffectivePolicy filter(Set<String> policyVocabularyNamespaces) {
        if (policyVocabularyNamespaces == null) {
            return this;
        }
        PolicyExpression filteredExpression = expression.filterExpression(policyVocabularyNamespaces);
        if (filteredExpression != null) {
            return new MessagePolicy(filteredExpression);
        } else {
            return null;
        }
    }

}