/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.policy.rt;

import java.util.Set;

import org.bluestemsoftware.specification.eoa.component.engine.rt.ServiceReference;
import org.bluestemsoftware.specification.eoa.component.service.Service;

/**
 * Aggregates {@link ElementPolicy} with policy subject SERVICE. Policy attached to the
 * following components imply policy subject SERVICE:
 * <p>
 * {@link Service}
 * <p>
 * {@link ServiceReference}
 */
public final class ServicePolicy extends EffectivePolicy {

    public ServicePolicy(PolicyExpression expression) {
        super(expression);
    }

    @Override
    public PolicySubject getPolicySubject() {
        return PolicySubject.SERVICE;
    }

    @Override
    public EffectivePolicy filter(Set<String> policyVocabularyNamespaces) {
        if (policyVocabularyNamespaces == null) {
            return this;
        }
        PolicyExpression filteredExpression = expression.filterExpression(policyVocabularyNamespaces);
        if (filteredExpression != null) {
            return new ServicePolicy(filteredExpression);
        } else {
            return null;
        }
    }

}