/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.rt;


/**
 * Models a 'nested' runtime <code>Component</code>, i.e. a runtime <code>Component</code> (child) which
 * is nested within another runtime <code>Component</code> (parent).
 */
public interface NestedComponentRT<R extends RootComponentRT, P extends ComponentRT> extends ComponentRT {

       /**
     * Retrieves parent component, i.e. component within which this component is 'nested.'
     * 
     * @return The parent component.
     */
    public P getParent();

    /**
     * Convenience method which returns top-level runtime component within which this runtime component is
     * nested.
     * <p>
     * Note that if runtime component is nested directly beneath a top-level component, then this
     * method returns same result as {@link #getParent()}.
     * 
     * @return RootComponentRT
     * @throws IllegalStateException
     *         If <code>RootComponent</code> is undefined.
     */
    public R getRootComponent();
    
}    
