/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.service;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.component.ComponentReference;
import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier;
import org.bluestemsoftware.specification.eoa.component.NestedComponent;
import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier.Scheme;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentName;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentType;
import org.bluestemsoftware.specification.eoa.component.binding.Binding;

public final class BindingReference extends NestedComponent<Service, Endpoint> implements ComponentReference<Binding> {

    private ComponentName bindingName;
    private Binding binding;

    public BindingReference(ComponentName bindingName) {
        if (bindingName == null) {
            throw new IllegalArgumentException("bindingName null");
        } else {
            this.bindingName = bindingName;
        }
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.ComponentReference#getReferencedComponentName()
     */
    public ComponentName getReferencedComponentName() {
        return bindingName;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.ComponentReference#getReferencedComponentType()
     */
    public ComponentType getReferencedComponentType() {
        return ComponentType.BINDING;
    }
    
    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.ComponentReference#getReferencedComponentClass()
     */
    public Class<Binding> getReferencedComponentClass() {
        return Binding.class;
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.ComponentReference#getReferencedComponent()
     */
    public Binding getReferencedComponent() {
        return binding;
    }

    final void establishReference() throws DeploymentException {
        if (binding != null) {
            return;
        }
        if (parent == null) {
            throw new IllegalStateException("parent null");
        }
        binding = getRootComponent().getComponentContext().findReferencedComponent(this);
        if (binding == null) {
            throw new DeploymentException("ComponentReference '"
                    + getFragmentIdentifier()
                    + "' is invalid. Referenced binding "
                    + bindingName
                    + " is undefined.");
        }
    }

    /*
     * (non-Javadoc)
     * @see org.bluestemsoftware.specification.eoa.component.Component#getFragmentIdentifier()
     */
    public final FragmentIdentifier getFragmentIdentifier() {
        if (fragmentIdentifier == null) {
            QName rootName = getRootComponent().getName();
            String pointerPart = parent.getFragmentIdentifier().getPointerPart();
            fragmentIdentifier = new FragmentIdentifier(rootName.getNamespaceURI(), Scheme.EOA,
                    "bindingReference", pointerPart.toString());
        }
        return fragmentIdentifier;
    }

    void setParent(Endpoint parent) {
        this.parent = parent;
    }

    // *********************** Identity Methods ************************************

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("bindingName: ");
        sb.append(System.getProperty("line.separator"));
        sb.append(bindingName);
        return sb.toString();
    }

    @Override
    public final int hashCode() {
        return hashCode(bindingName);
    }

    @Override
    public final boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }

        if (!(obj instanceof BindingReference)) {
            return false;
        }

        if ((bindingName == ((BindingReference)obj).bindingName || (bindingName != null && bindingName
                .equals(((BindingReference)obj).bindingName)))

        ) {
            return true;
        }

        return false;
    }

    private int hashCode(Object object) {
        return (object == null ? 0 : object.hashCode());
    }

}
