/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.types;

import javax.xml.namespace.QName;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier;
import org.bluestemsoftware.specification.eoa.component.FragmentIdentifier.Scheme;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentName;

/**
 * Models a <code>Schema</code> include component, which identifies a dependency between
 * parent <code>Schema</code> and <code>Schema</code> identified by qualified name. The
 * included schema MUST be same type, i.e. language as parent schema.
 */
public final class SchemaInclude extends SchemaReference {

    public SchemaInclude(ComponentName schemaName, String location) {
        super(schemaName, location);
    }
    
    public FragmentIdentifier getFragmentIdentifier() {
        if (fragmentIdentifier == null) {
            QName rootName = getRootComponent().getName();
            StringBuffer pointerPart = new StringBuffer();
            pointerPart.append(rootName.getLocalPart());
            pointerPart.append("/");
            pointerPart.append(location);
            fragmentIdentifier = new FragmentIdentifier(rootName.getNamespaceURI(), Scheme.EOA, "schemaInclude",
                    pointerPart.toString());
        }
        return fragmentIdentifier;
    }
    
    void validate() throws DeploymentException {
    }

    // *********************** Identity Methods ************************************

    @Override
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("location: ");
        sb.append(location);
        return sb.toString();
    }

    @Override
    public int hashCode() {
        return hashCode(location);
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        }

        if (!(obj instanceof SchemaInclude)) {
            return false;
        }

        if ((location == ((SchemaInclude)obj).location || (location != null && location
                .equals(((SchemaInclude)obj).location)))) {
            return true;
        }

        return false;
    }

    private int hashCode(Object object) {
        return (object == null ? 0 : object.hashCode());
    }

}
