/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.component.types.rt;

import java.util.List;

import org.bluestemsoftware.specification.eoa.ext.RootExtension;
import org.bluestemsoftware.specification.eoa.ext.RootExtension.AbstractRootExtension;
import org.bluestemsoftware.specification.eoa.ext.expression.ExpressionFactory;
import org.bluestemsoftware.specification.eoa.ext.expression.EvaluationException;
import org.w3c.dom.Node;

public abstract class ExpressionRT extends AbstractRootExtension {

    /*
     * (non-Javadoc)
     * 
     * SPI
     */
    public interface Provider extends RootExtension.Provider {
    }

    public ExpressionRT(ExpressionFactory factory, Provider provider) {
        super(factory, provider);
    }

    /**
     * Declares a namespace prefix used within expression query.
     * 
     * @param prefix
     * @param namespace
     */
    public abstract void addNamespace(String prefix, String namespace);

    /**
     * Evaluates <code>Document</code> against this <code>Expression</code> according to
     * language specific rules and returns result as a <code>String</code>.
     * <p>
     * Note that the underlying implementation of this method MUST be threadsafe, i.e. capable
     * of supporting concurrent use by multiple threads.
     * 
     * @param node
     *        against which this <code>Expression</code> is evaluated.
     * @return String
     * @throws EvaluationException
     *         if an error occurs evaluating expression.
     */
    public abstract String evaluateString(Node node) throws EvaluationException;

    /**
     * Evaluates <code>Document</code> against this <code>Expression</code> according to
     * language specific rules and returns result as a <code>Number</code>.
     * <p>
     * Note that the underlying implementation of this method MUST be threadsafe, i.e. capable
     * of supporting concurrent use by multiple threads.
     * 
     * @param node
     *        against which this <code>Expression</code> is evaluated.
     * @return Number
     * @throws EvaluationException
     *         if an error occurs evaluating expression.
     */
    public abstract Number evaluateNumber(Node node) throws EvaluationException;

    /**
     * Evaluates <code>Document</code> against this <code>Expression</code> according to
     * language specific rules and returns result as a <code>Boolean</code>.
     * <p>
     * Note that the underlying implementation of this method MUST be threadsafe, i.e. capable
     * of supporting concurrent use by multiple threads.
     * 
     * @param node
     *        against which this <code>Expression</code> is evaluated.
     * @return Boolean
     * @throws EvaluationException
     *         if an error occurs evaluating expression.
     */
    public abstract Boolean evaluateBoolean(Node node) throws EvaluationException;

    /**
     * Evaluates <code>Document</code> against this <code>Expression</code> according to
     * language specific rules and returns result as a <code>Node</code>.
     * <p>
     * Note that the underlying implementation of this method MUST be threadsafe, i.e. capable
     * of supporting concurrent use by multiple threads.
     * 
     * @param node
     *        against which this <code>Expression</code> is evaluated.
     * @return Node
     * @throws EvaluationException
     *         if an error occurs evaluating expression.
     */
    public abstract Node evaluateNode(Node node) throws EvaluationException;

    /**
     * Evaluates <code>Document</code> against this <code>Expression</code> according to
     * language specific rules and returns result as a <code>Set</code>.
     * <p>
     * Note that the underlying implementation of this method MUST be threadsafe, i.e. capable
     * of supporting concurrent use by multiple threads.
     * 
     * @param node
     *        against which this <code>Expression</code> is evaluated.
     * @return Set
     * @throws EvaluationException
     *         if an error occurs evaluating expression.
     */
    public abstract List<Node> evaluateNodes(Node node) throws EvaluationException;

}
