/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext;

/**
 * Models a 'top-level' <code>Extension</code> whose definition is loaded by
 * <code>System</code> using a type specific instance of {@link ExtensionFactory}.
 */

public interface RootExtension extends Extension {

    public interface Provider extends Extension.Provider {
    }

    /**
     * Returns a reference URI which identifies <code>Extension</code> type within context of
     * a specific extension category, e.g. ENGINE, FEATURE, POLICY, etc ...
     * <p>
     * Note that value SHOULD be a well-known URI which associates instance with specification
     * implemented by <code>Extension</code>, e.g. 'http://www.w3.org/ns/wsdl/soap',
     * 'http://www.w3.org/ns/ws-policy', etc ...
     * 
     * @return ref URI which identifies <code>Extension</code> type.
     * 
     * @see ExtensionFactory#getType()
     */
    public String getExtensionType();

    /**
     * Abstract <code>RootExtension</code> object which MAY be extended by a concrete
     * instance of <code>RootExtension</code>.
     */
    public abstract class AbstractRootExtension implements RootExtension {

        protected String extensionType;
        protected ExtensionFactory factory;
        protected Provider provider;

        public AbstractRootExtension(ExtensionFactory factory, Provider provider) {
            if (factory == null) {
                throw new IllegalArgumentException("extensionFactory null");
            } else {
                this.factory = factory;
                this.extensionType = factory.getExtensionType();
            }
            if (provider == null) {
                throw new IllegalArgumentException("provider not set");
            } else {
                this.provider = provider;
                this.provider.spi_setConsumer(this);
            }
        }

        /*
         * (non-Javadoc)
         * @see org.bluestemsoftware.specification.eoa.ext.RootExtension#getExtensionType()
         */
        public String getExtensionType() {
            return extensionType;
        }

        /*
         * (non-Javadoc)
         * @see org.bluestemsoftware.specification.eoa.ext.Extension#getExtensionFactory()
         */
        public ExtensionFactory getExtensionFactory() {
            return factory;
        }

        /*
         * (non-Javadoc)
         * @see org.bluestemsoftware.specification.eoa.ext.Extension#getExtensionProvider()
         */
        public org.bluestemsoftware.specification.eoa.ext.Extension.Provider getExtensionProvider() {
            return provider;
        }

    }

}
