/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.appdata;

import org.bluestemsoftware.specification.eoa.component.message.rt.Content;

/**
 * Models 'application data' transmitted 'out-of-band', i.e. outside of the normal data
 * channel, e.g. via the soap:Body. Note that data is 'abstract', i.e. protocol specific
 * attributes SHOULD NOT be sent/received via application data.
 */
public interface ApplicationData extends Content {

    /**
     * A protocol agnostic getter, used to retrieve targeted role, i.e. role performed by
     * service to which data is targeted.
     * 
     * @return role URI or <code>null</code> if undefined.
     */
    public String getRole();

    /**
     * A protocol agnostic setter, used to set targeted role, i.e. role performed by service to
     * which data is targeted.
     * 
     * @param role
     *        URI
     */
    public void setRole(String role);

    /**
     * A protocol agnostic getter, used to retrieve 'must understand' flag, i.e. which is an
     * indication that targeted service MUST understand semantics implied by this data.
     * 
     * @param mustUnderstand
     */
    public void setMustUnderstand(boolean mustUnderstand);

    /**
     * A protocol agnostic setter, used to set 'must understand' flag, i.e. which is an
     * indication that targeted service MUST understand semantics implied by this data.
     * 
     * @return
     */
    public boolean mustUnderstand();

}
