/**
 * Copyright 2008 Bluestem Software LLC
 *   
 * All Rights Reserved. Bluestem Software LLC licenses this file
 * to you under the EOA Specification License Agreement [1]. You
 * may not use this file except in compliance with the License. You
 * may obtain a copy of the License at:
 *   
 * [1] http://alakai.org/licensing/specification-eoa-license.html
 *   
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

package org.bluestemsoftware.specification.eoa.ext.feature.ws.transport;

import org.bluestemsoftware.specification.eoa.DeploymentException;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.rt.TransportProtocolRT;

/**
 * Statically describes a 'Transport Protocol' which affects runtime transport behavior and
 * which MUST be implemented by concrete instances of {@link UnderlyingProtocol}.
 * 
 * @see TransportProtocolRT
 */
public interface TransportProtocol {

    /**
     * Using indicated path, query and fragment, creates a protocol specific
     * <code>Endpoint</code> address for a locally hosted <code>Service</code>.
     * <p>
     * Note that the rules used to construct an address are defined by protocol specific
     * instances of <code>TransportProtocol</code>.
     * 
     * @param path
     * @param query
     * @param fragment
     * @return
     * @throws DeploymentException
     */
    public String createEndpointAddress(String path, String query, String fragment) throws DeploymentException;

    
}
